/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsLogReport.h
    
Abstract:

    Data definition for report library

--*/

#ifndef __EMS_REPORT_H__
#define __EMS_REPORT_H__

#include <EmsTypes.h>

#define EMS_REPORT_DIR      "Report"

#define EMS_INDEX_LEN       20
#define EMS_DESCRIPTION_LEN 1000
#define EMS_GUID_LEN        40
#define EMS_NAME_LEN        100

#define EMS_FILE_BUFFER_LEN 100000

typedef struct _EMS_CASE_INFOR {
  struct _EMS_CASE_INFOR  *Next;
  struct _EMS_CASE_INFOR  *Prev;
  INT8                    Index[EMS_INDEX_LEN];
  INT8                    TestName[EMS_NAME_LEN];
  INT8                    TestDescription[EMS_DESCRIPTION_LEN];
  INT8                    Guid[EMS_GUID_LEN];
} EMS_CASE_INFOR;

typedef struct _EMS_REPORT_ITEM {
  struct _EMS_REPORT_ITEM *Next;
  struct _EMS_REPORT_ITEM *Prev;
  UINT32                  PassNumber;
  UINT32                  WarnNumber;
  UINT32                  FailNumber;
  INT8                    TestCategory[EMS_NAME_LEN];
  EMS_CASE_INFOR          *FailCase;
  EMS_CASE_INFOR          *PassCase;
} EMS_REPORT_ITEM;

typedef struct {
  UINT32          TotalPass;
  UINT32          TotalWarn;
  UINT32          TotalFail;
  EMS_REPORT_ITEM *ReportItem;
} EMS_REPORT_INFOR;

typedef struct _EMS_REPORT_GUID {
  struct _EMS_REPORT_GUID *Next;
  struct _EMS_REPORT_GUID *Prev;
  UINT32                  GuidNumber;
  INT8                    Guid[EMS_GUID_LEN];
} EMS_REPORT_GUID;


//
// EMS library GenerateReport API for assertions
//
EFI_STATUS
GenerateReport (
  INT8               *ReportName
  )
/*++

Routine Description:

  Generate the final log report for assertion

Arguments:

  ReportName  - the report file name

Returns:

  EFI_SUCCESS - Generate the final log report successfully
  OTHERS        - Something failed.

--*/
;

//
// EMS library GenerateReportCase API for cases
//
EFI_STATUS
GenerateReportCase (
  INT8               *ReportName
  )
/*++

Routine Description:

  Generate the final log report for case

Arguments:

  ReportName  - the report file name

Returns:

  EFI_SUCCESS - Generate the final log report successfully
  OTHERS        - Something failed.

--*/
;

#endif // __EMS_REPORT_H__
