/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsProtocols.c
    
Abstract:

    Implementation for protocol related common functions

--*/

#include <EmsUtilityString.h>

#include "EmsProtocols.h"
#include "EmsProtoEth.h"
#include "EmsProtoIp.h"
#include "EmsProtoIgmp.h"
#include "EmsProtoArp.h"
#include "EmsProtoUdp.h"
#include "EmsProtoIcmp.h"
#include "EmsProtoDhcp.h"
#include "EmsProtoTcp.h"

PROTOCOL_ENTRY_T  *Protocols[] = {
  &EthProtocol,
  &IpProtocol,
  &IgmpProtocol,
  &ArpProtocol,
  &UdpProtocol,
  &IcmpProtocol,
  &DhcpProtocol,
  &TcpProtocol,
  NULL
};

PROTOCOL_ENTRY_T *
GetProtocolByName (
  IN INT8 *Name
  )
/*++

Routine Description:

  Get the protocol entry by specified name

Arguments:

  Name  - Protocl name

Returns:

  The protocol entry or NULL.

--*/
{
  UINT32  Index;

  for (Index = 0; Protocols[Index]; Index++) {
    if (0 == strcmp_i (Protocols[Index]->Name, Name)) {
      return Protocols[Index];
    }
  }

  return NULL;
}

VOID_P
EmsFieldDump (
  IN OUT  INT8     *Buff,
  IN      FIELD_T  *Fields
  )
/*++

Routine Description:

  Dump the field information into a specified buffer.

Arguments:

  Buff    - The buffer for the fields
  Fields  - The pointer to the fields

Returns:

  None

--*/
{
  UINT32  Index;
  INT8    Log[100];
  Buff[0] = '\0';
  for (Index = 0; Fields[Index].Name; Index++) {
    strcat (Buff, Fields[Index].Name);
    switch (Fields[Index].Type) {
    case OCTET1:
      sprintf (Log, "=0x%02x ", *(UINT8 *) Fields[Index].Value);
      break;

    case OCTET2:
      sprintf (Log, "=0x%04x ", *(UINT16 *) Fields[Index].Value);
      break;

    case OCTET4:
      sprintf (Log, "=0x%08x ", *(UINT32 *) Fields[Index].Value);
      break;

    case IPADDR:
      Log[0] = '=';
      Ipv4ToAsciiString (&Log[1], *(UINT32 *) Fields[Index].Value);
      break;

    case MACADDR:
      Log[0] = '=';
      MacToAsciiString (&Log[1], ((UINT8 *) Fields[Index].Value));
      break;

    case STRING:
      Log[0] = '=';
      sprintf (Log, "=%s ", *((UINT8 **) Fields[Index].Value));
      break;

    default:
      sprintf (Log, " ");
      break;
    }

    strcat (Buff, Log);
  }

  return ;
}
