/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsPktCapture.h
    
Abstract:

    Data definition for packet capture function

--*/

#ifndef __EMS_CAPTURE_H__
#define __EMS_CAPTURE_H__

#include <EmsTypes.h>
#include <EmsTclInit.h>
#include <EmsNet.h>

#include "EmsVtcpNamedList.h"

extern
VOID_P
CaptureTclInit (
  IN Tcl_Interp *Interp
  )
/*++

Routine Description:

  Packet capture related TCL initialization.

Arguments:

  Interp      - TCL intepreter.

Returns:

  None.

--*/
;

//
// This struct is used for capturing one packet.
// Transfer between the parent and child processes
//
typedef struct {
  BOOLEAN         GotOne;   // Capture one packet?
  UINT8           *Packet;  // Packet content
  UINT32          Len;      // The length of the packet
  struct timeval  Time;     // When got the packet?
} CAP_INFO;

//
// This struct is used for capturing multiple packets
// Transfer between the parent and child processes
// Generally, we captures multiple packets only in performance test.
//
typedef struct {
  UINT32  Count;  // Only the number will be return from chile process
} MULTI_CAP_INFO;

#define MAX_FILTER_LEN  256

//
// This struct is used for defining the capture control block
//
typedef struct {
  UINT32          Received;     // Received
  UINT8           *ReceiveBuff; // Buffer the lasted packet received
  UINT32          ReceiveLen;   // the length of the received packet
  pcap_t          *Capturer;    //
  UINT8           CaptureFilter[MAX_FILTER_LEN];
  struct timeval  Time;
} CCB;

extern NL_LIST  CcbList;

VOID
EmsCaptureEndAll(
  VOID
  )
/*++

Routine Description:

  Close all the EMS capture

Arguments:

  None

Returns:

  None

--*/
;

#endif // _EMS_CAPTURE_H_
