/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsTest.h
    
Abstract:

  Incude header files for EMS Test functions

--*/

#ifndef __EMS_TEST_H__
#define __EMS_TEST_H__

#include "tcl.h"
#include "EmsTypes.h"

typedef struct _TESTCASE_LIST_ENTRY {
  struct _TESTCASE_LIST_ENTRY *Next;
  UINT8                       CasePath[256];
  UINT8                       CaseName[64];
} TESTCASE_LIST_ENTRY;

typedef struct _TESTCASE_LIST_HEAD {
  TESTCASE_LIST_ENTRY *Head;
  TESTCASE_LIST_ENTRY *Tail;
  UINT32              EntryNum;
} TESTCASE_LIST_HEAD;

STATIC Tcl_CmdProc  EmsTestInit;
STATIC Tcl_CmdProc  EmsTestCaseScriptEval;
STATIC Tcl_CmdProc  EmsRemoteValidationTestStart;
STATIC Tcl_CmdProc  EmsRemoteValidationTestStop;
STATIC Tcl_CmdProc  EmsRemoteValidationTestContinue;
STATIC Tcl_CmdProc  EmsTestIsRunning;
STATIC Tcl_CmdProc  EmsThreadCmd;
STATIC Tcl_CmdProc  RemoteValidationTestCaseAllPost;

INT32
EmsTestInit (
  IN ClientData        clientData,
  IN Tcl_Interp        *Interp,
  IN INT32             Argc,
  IN CONST84 INT8      *Argv[]
  )
/*++

Routine Description:

  TCL command "EmsTestInit" implementation routine  

Arguments:

  clientData  - Private data, if any.
  Interp      - TCL intepreter
  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  TCL_OK or TCL_ERROR

--*/
;

INT32
EmsThreadCmd (
  IN ClientData        clientData,
  IN Tcl_Interp        *Interp,
  IN INT32             Argc,
  IN CONST84 INT8      *Argv[]
  )
/*++

Routine Description:

  TCL command "EmsThreadCmd" implementation routine  

Arguments:

  clientData  - Private data, if any.
  Interp      - TCL intepreter
  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  TCL_OK or TCL_ERROR

--*/
;

INT32
EmsTestCaseScriptEval (
  IN ClientData        clientData,
  IN Tcl_Interp        *Interp,
  IN INT32             Argc,
  IN CONST84 INT8      *Argv[]
  )
/*++

Routine Description:

  TCL command "EmsTestCaseScriptEval" implementation routine  

Arguments:

  clientData  - Private data, if any.
  Interp      - TCL intepreter
  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  TCL_OK or TCL_ERROR

--*/
;

INT32
EmsRemoteValidationTestStop (
  IN ClientData        clientData,
  IN Tcl_Interp        *Interp,
  IN INT32             Argc,
  IN CONST84 INT8      *Argv[]
  )
/*++

Routine Description:

  TCL command "EmsRemoteValidationTestStop" implementation routine  

Arguments:

  clientData  - Private data, if any.
  Interp      - TCL intepreter
  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  TCL_OK

--*/
;

INT32
EmsRemoteValidationTestStart (
  IN ClientData        clientData,
  IN Tcl_Interp        *Interp,
  IN INT32             Argc,
  IN CONST84 INT8      *Argv[]
  )
/*++

Routine Description:

  TCL command "EmsRemoteValidationTestStart" implementation routine  

Arguments:

  clientData  - Private data, if any.
  Interp      - TCL intepreter
  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  TCL_OK

--*/
;

INT32
EmsTestIsRunning (
  IN ClientData        clientData,
  IN Tcl_Interp        *Interp,
  IN INT32             Argc,
  IN CONST84 INT8      *Argv[]
  )
/*++

Routine Description:

  TCL command "EmsTestIsRunning" implementation routine  

Arguments:

  clientData  - Private data, if any.
  Interp      - TCL intepreter
  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  TCL_OK

--*/
;

VOID
EmsTestTclInit (
  IN Tcl_Interp *Interp
  )
/*++

Routine Description:

  EMS test related TCL initialization.

Arguments:

  Interp      - TCL intepreter.

Returns:

  None

--*/
;

#endif
