/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/**
 *  This file is a test image for the Image Boot Services Test
 */
/*++

Module Name:
    Application4.c

Abstract:       
    for Image Boot Services Black Box Test

--*/

#include "ProtocolDefinition.h"
#include "Misc.h"
#include "EfiTestLib.h"

//
// data definition here
//

EFI_STATUS
InitializeEfiApplication4 (
  IN EFI_HANDLE           ImageHandle,
  IN EFI_SYSTEM_TABLE     *SystemTable
  );

EFI_APPLICATION_ENTRY_POINT(InitializeEfiApplication4)

EFI_STATUS
InitializeEfiApplication4 (
  IN EFI_HANDLE           ImageHandle,
  IN EFI_SYSTEM_TABLE     *SystemTable
  )
{
  EFI_STATUS              Status;    
  EFI_HANDLE              *HandleBuffer;
  UINTN                   NoHandles;
  CHAR16                  *ExitData;
  CHAR16                  *ErrorMessage;
  UINTN                   ExitSize;
   
  HandleBuffer = NULL;
  ExitData = NULL;
  ExitSize = 0;
  
  EfiInitializeTestLib (ImageHandle, SystemTable);
  
  gtBS->InstallProtocolInterface (
                      &ImageHandle,
                      &mImageTestNoInterfaceProtocol3Guid,
                      EFI_NATIVE_INTERFACE,
                      NULL
                      );
                      
  Status = gtBS->LocateHandleBuffer (
                  ByProtocol,
                  &mImageTestNoInterfaceProtocol4Guid,
                  NULL,
                  &NoHandles,
                  &HandleBuffer
                  );
  if (EFI_ERROR(Status)) {
    goto Done;
  }
  
  gtBS->OpenProtocol (
                HandleBuffer[0],
                &mImageTestNoInterfaceProtocol4Guid,
                NULL,
                ImageHandle,
                NULL,
                EFI_OPEN_PROTOCOL_TEST_PROTOCOL
                );
                
Done:
  
   
  if (HandleBuffer != NULL && NoHandles > 0) {
    gtBS->FreePool (HandleBuffer);
  }
  
  gtBS->UninstallProtocolInterface (
                        ImageHandle,
                        &mImageTestNoInterfaceProtocol3Guid,
                        NULL
                        );  
                        
  ErrorMessage = ERROR_MESSAGE_FOR_APPLICATION_4;
  
  ExitSize = StrSize (ErrorMessage);
  
  Status = gtBS->AllocatePool (
                        EfiBootServicesData,
                        ExitSize,
                        (VOID**)&ExitData
                        );
  if (!EFI_ERROR(Status)) {
    gtBS->CopyMem (ExitData, ErrorMessage, ExitSize);
  }
  
  gtBS->Exit (ImageHandle,EFI_DEVICE_ERROR,ExitSize,ExitData);

  //
  //the code should not arrive here so we add a variable to verify if the Exit
  //return control to StartImage correctly.
  //
  
  //
  //first make sure there is no duplicated variable exist so we first delete once.
  //
  gtRT->SetVariable (
          IMAGE_APPLICATION4_CALLED_AFTER_EXIT,
          &mImageTestVariableVendorGuid,
          EFI_VARIABLE_BOOTSERVICE_ACCESS | EFI_VARIABLE_RUNTIME_ACCESS,
          0,
          (VOID*)&ImageHandle
          );
  
  //
  //then call SetVariable to set a variable.
  //
           
  gtRT->SetVariable (
          IMAGE_APPLICATION4_CALLED_AFTER_EXIT,
          &mImageTestVariableVendorGuid,
          EFI_VARIABLE_BOOTSERVICE_ACCESS | EFI_VARIABLE_RUNTIME_ACCESS,
          sizeof(EFI_HANDLE),
          (VOID*)&ImageHandle
          );

  //
  // actually should not arrive here. Just to pass compiling.
  //
  return EFI_SUCCESS;
}

