/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  AcpiTableBBTestMain.c

Abstract:

  Test Driver of Acpi Table Protocol

--*/

#include "AcpiTableProtocolBBTestMain.h"

//
// Global variables
//

EFI_HANDLE mImageHandle;

EFI_DRIVER_ENTRY_POINT(InitializeBBTestAcpiTableProtocol)

EFI_BB_TEST_PROTOCOL_FIELD gBBTestProtocolField = {
  EFI_ACPI_TABLE_PROTOCOL_TEST_REVISION,
  EFI_ACPI_TABLE_PROTOCOL_GUID,
  L"Acpi Table Protocol Test",
  L"UEFI 2.1 Acpi Table Protocol Black-Box Test"
};

EFI_GUID gSupportProtocolGuid[] = {
  EFI_STANDARD_TEST_LIBRARY_GUID,
  EFI_NULL_GUID
};

EFI_BB_TEST_ENTRY_FIELD gBBTestEntryField[] = {
  {
    EFI_ACPI_TABLE_PROTOCOL_TEST_ENTRY_GUID0101,
    L"InstallAcpiTableFunction",
    L"Function test for Acpi Table Protocol InstallAcpiTable().",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestInstallAcpiTableFunctionTest
  },
  {
    EFI_ACPI_TABLE_PROTOCOL_TEST_ENTRY_GUID0102,
    L"UninstallAcpiTableFunction",
    L"Function test for Acpi Table Protocol UninstallAcpiTable().",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestUninstallAcpiTableFunctionTest
  },
  {
    EFI_ACPI_TABLE_PROTOCOL_TEST_ENTRY_GUID0201,
    L"InstallAcpiTableConformance",
    L"Conformance test for Acpi Table Protocol InstallAcpiTable().",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestInstallAcpiTableConformanceTest
  },
  {
    EFI_ACPI_TABLE_PROTOCOL_TEST_ENTRY_GUID0202,
    L"UninstallAcpiTableConformance",
    L"Conformance test for Acpi Table Protocol UninstallAcpiTable( ).",
    EFI_TEST_LEVEL_MINIMAL,
    gSupportProtocolGuid,
    EFI_TEST_CASE_AUTO,
    BBTestUninstallAcpiTableConformanceTest
  },
  0
};

EFI_BB_TEST_PROTOCOL                  *gBBTestProtocolInterface;

//
// Unload function
//

EFI_STATUS
UnloadAcpiTableProtocolBBTest (
  IN EFI_HANDLE                   ImageHandle
  );


EFI_STATUS
InitializeBBTestAcpiTableProtocol (
  IN EFI_HANDLE                   ImageHandle,
  IN EFI_SYSTEM_TABLE             *SystemTable
  )
{
  EfiInitializeTestLib (ImageHandle, SystemTable);

  //
  // initialize test utility lib
  //

  InitializeLib (ImageHandle, SystemTable);

  mImageHandle = ImageHandle;

  return EfiInitAndInstallBBTestInterface (
           &ImageHandle,
           &gBBTestProtocolField,
           gBBTestEntryField,
           UnloadBBTestAcpiTableProtocol,
           &gBBTestProtocolInterface
           );
}


EFI_STATUS
UnloadBBTestAcpiTableProtocol (
  IN EFI_HANDLE                   ImageHandle
  )
{
  return EfiUninstallAndFreeBBTestInterface (
           ImageHandle,
           gBBTestProtocolInterface
           );
}
